@@ -18,16 +18,16 @@
 extern int GdmFirstVT;
 extern gboolean GdmVTAllocation;
 
-#if defined (__linux__) || defined (__FreeBSD__)
+#if defined (__linux__) || defined (__FreeBSD__) || defined(__DragonFly__)
 
 #ifdef __linux__
 #include <sys/vt.h>
 #endif
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__DragonFly__)
 #include <sys/consio.h>
 #endif
 
-#ifdef __FreeBSD__
+#if defined( __FreeBSD__) || defined(__DragonFly__)
 static const char*
 __itovty(int val)
 {
@@ -51,7 +51,7 @@ open_vt (int vtno)
 
 #if defined (__linux__)
 	vtname = g_strdup_printf ("/dev/tty%d", vtno);
-#elif defined (__FreeBSD__)
+#elif defined (__FreeBSD__) || defined(__DragonFly__)
 	vtname = g_strdup_printf ("/dev/ttyv%s", __itovty(vtno - 1));
 #endif
 	do {
@@ -104,7 +104,7 @@ get_free_vt (int *vtfd)
 
 #if defined (__linux__)
 		if ((ioctl(fd, VT_OPENQRY, &vtno) < 0) || (vtno == -1)) {
-#elif defined (__FreeBSD__)
+#elif defined (__FreeBSD__) || defined(__DragonFly__)
 		if (ioctl(fd, VT_OPENQRY, &vtno) == -1) {
 #endif
 			vtno = -1;
@@ -177,7 +177,7 @@ gdm_get_cur_vt (void)
 {
 #if defined (__linux__)
 	struct vt_stat s;
-#elif defined (__FreeBSD__)
+#elif defined (__FreeBSD__) || defined(__DragonFly__)
 	int vtno;
 #endif
 	int fd;
@@ -203,7 +203,7 @@ gdm_get_cur_vt (void)
 	*/
 
 	return s.v_active;
-#elif defined (__FreeBSD__)
+#elif defined (__FreeBSD__) || defined(__DragonFly__)
 	if (ioctl (fd, VT_GETACTIVE, &vtno) == -1) {
 		VE_IGNORE_EINTR (close (fd));
 		return -1;
