@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -276,11 +276,20 @@ gdm_server_stop (GdmDisplay *disp)
     if (disp->dsp != NULL) {
 	    /* on XDMCP servers first kill everything in sight */
 	    if (disp->type == TYPE_XDMCP)
-		    gdm_server_whack_clients (d);
+		    gdm_server_whack_clients (disp->dsp);
 	    XCloseDisplay (disp->dsp);
 	    disp->dsp = NULL;
     }
 
+    /* Kill our parent connection if one existed */
+    if (disp->parent_dsp != NULL) {
+	    /* on XDMCP servers first kill everything in sight */
+	    if (disp->type == TYPE_XDMCP_PROXY)
+		    gdm_server_whack_clients (disp->parent_dsp);
+	    XCloseDisplay (disp->parent_dsp);
+	    disp->parent_dsp = NULL;
+    }
+
     if (disp->servpid <= 0)
 	    return;
 
@@ -339,9 +348,10 @@ busy_ask_user (GdmDisplay *disp)
     if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) == 0) {
 	    char *error = g_strdup_printf
 		    (C_(N_("There already appears to be an X server "
-			   "running on display %s.  Should I try another "
-			   "display number?  If you answer no, I will "
-			   "attempt to start the server on %s again.%s")),
+			   "running on display %s.  Should another "
+			   "display number by tried?  Answering no will "
+			   "cause GDM to attempt starting the server "
+			   "on %s again.%s")),
 		     disp->name,
 		     disp->name,
 #ifdef __linux__
@@ -368,7 +378,7 @@ busy_ask_user (GdmDisplay *disp)
 
 /* Checks only output, no XFree86 v4 logfile */
 static gboolean
-display_xnest_no_connect (GdmDisplay *disp)
+display_parent_no_connect (GdmDisplay *disp)
 {
 	char *logname = gdm_make_filename (GdmLogDir, d->name, ".log");
 	FILE *fp;
@@ -392,7 +402,7 @@ display_xnest_no_connect (GdmDisplay *disp)
 		 * of course additions are welcome to make this more robust */
 		if (strstr (buf, "Unable to open display \"") == buf) {
 			gdm_error (_("Display '%s' cannot be opened by Xnest"),
-				   ve_sure_string (disp->xnest_disp));
+				   ve_sure_string (disp->parent_disp));
 			VE_IGNORE_EINTR (fclose (fp));
 			return TRUE;
 		}
@@ -553,7 +563,7 @@ do_server_wait (GdmDisplay *d)
 {
     /* Wait for X server to send ready signal */
     if (d->servstat == SERVER_PENDING) {
-	    if (d->server_uid != 0 && ! d->handled) {
+	    if (d->server_uid != 0 && ! d->handled && ! d->chosen_hostname) {
 		    /* FIXME: If not handled, we just don't know, so
 		     * just wait a few seconds and hope things just work,
 		     * fortunately there is no such case yet and probably
@@ -661,6 +671,45 @@ do_server_wait (GdmDisplay *d)
     }
 }
 
+/* We keep a connection (parent_dsp) open with the parent X server
+ * before running a proxy on it to prevent the X server resetting
+ * as we open and close other connections.
+ * Note that XDMCP servers, by default, reset when the seed X
+ * connection closes whereas usually the X server only quits when
+ * all X connections have closed.
+ */
+static gboolean
+connect_to_parent (GdmDisplay *d)
+{
+	int maxtries;
+	int openretries;
+
+	gdm_debug ("gdm_server_start: Connecting to parent display \'%s\'",
+		   d->parent_disp);
+
+	d->parent_dsp = NULL;
+
+	maxtries = SERVER_IS_XDMCP (d) ? 10 : 2;
+
+	openretries = 0;
+	while (openretries < maxtries &&
+	       d->parent_dsp == NULL) {
+		d->parent_dsp = XOpenDisplay (d->parent_disp);
+
+		if G_UNLIKELY (d->parent_dsp == NULL) {
+			gdm_debug ("gdm_server_start: Sleeping %d on a retry", 1+openretries*2);
+			gdm_sleep_no_signal (1+openretries*2);
+			openretries++;
+		}
+	}
+
+	if (d->parent_dsp == NULL)
+		gdm_error (_("%s: failed to connect to parent display \'%s\'"),
+			   "gdm_server_start", d->parent_disp);
+
+	return d->parent_dsp != NULL;
+}
+
 /**
  * gdm_server_start:
  * @disp: Pointer to a GdmDisplay structure
@@ -688,7 +737,7 @@ gdm_server_start (GdmDisplay *disp,
     gdm_server_stop (d);
 
     /* First clear the VT number */
-    if (d->type == TYPE_LOCAL ||
+    if (d->type == TYPE_STATIC ||
 	d->type == TYPE_FLEXI) {
 	    d->vt = -1;
 	    gdm_slave_send_num (GDM_SOP_VT_NUM, -1);
@@ -707,6 +756,9 @@ gdm_server_start (GdmDisplay *disp,
 	    gdm_slave_send_num (GDM_SOP_DISP_NUM, flexi_disp);
     }
 
+    if (d->type == TYPE_XDMCP_PROXY &&
+	! connect_to_parent (d))
+	    return FALSE;
 
     gdm_debug ("gdm_server_start: %s", d->name);
 
@@ -714,6 +766,7 @@ gdm_server_start (GdmDisplay *disp,
     if ( ! gdm_auth_secure_display (d)) 
 	    return FALSE;
     gdm_slave_send_string (GDM_SOP_COOKIE, d->cookie);
+    gdm_slave_send_string (GDM_SOP_AUTHFILE, d->authfile);
     ve_setenv ("DISPLAY", d->name, TRUE);
 
     if ( ! setup_server_wait (d))
@@ -721,7 +774,7 @@ gdm_server_start (GdmDisplay *disp,
 
     d->servstat = SERVER_DEAD;
 
-    if (d->type == TYPE_LOCAL ||
+    if (d->type == TYPE_STATIC ||
 	d->type == TYPE_FLEXI) {
 	    vtarg = gdm_get_empty_vt_argument (&vtfd, &vt);
     }
@@ -753,7 +806,7 @@ gdm_server_start (GdmDisplay *disp,
 
 	    if (SERVER_IS_FLEXI (d))
 		    gdm_slave_send_num (GDM_SOP_FLEXI_OK, 0 /* bogus */);
-	    if (d->type == TYPE_LOCAL ||
+	    if (d->type == TYPE_STATIC ||
 		d->type == TYPE_FLEXI) {
 		    if (vt >= 0)
 			    d->vt = vt;
@@ -773,10 +826,10 @@ gdm_server_start (GdmDisplay *disp,
 	    break;
     }
 
-    if (disp->type == TYPE_FLEXI_XNEST &&
-	display_xnest_no_connect (disp)) {
+    if (SERVER_IS_PROXY (disp) &&
+	display_parent_no_connect (disp)) {
 	    gdm_slave_send_num (GDM_SOP_FLEXI_ERR,
-				5 /* Xnest can't connect */);
+				5 /* proxy can't connect */);
 	    _exit (DISPLAY_REMANAGE);
     }
 
@@ -845,7 +898,7 @@ void
 gdm_server_checklog (GdmDisplay *disp)
 {
 	if (d->vt < 0 &&
-	    (d->type == TYPE_LOCAL ||
+	    (d->type == TYPE_STATIC ||
 	     d->type == TYPE_FLEXI)) {
 		d->vt = display_vt (d);
 		if (d->vt >= 0)
@@ -932,7 +985,7 @@ gdm_server_resolve_command_line (GdmDisplay *disp,
 	} else if (bin[0] != '/') {
 		GdmXServer *svr = gdm_find_x_server (bin);
 		if (svr == NULL) {
-			gdm_error (_("Server name '%s' not found, "
+			gdm_error (_("Server name '%s' not found; "
 				     "using standard server"), bin);
 			argv = ve_split (GdmStandardXServer);
 		} else {
@@ -1145,27 +1198,36 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 	sigemptyset (&mask);
 	sigprocmask (SIG_SETMASK, &mask, NULL);
 
-	if (d->type == TYPE_FLEXI_XNEST) {
-		char *font_path = NULL;
-		ve_setenv ("DISPLAY", d->xnest_disp, TRUE);
-		if (d->xnest_auth_file != NULL)
-			ve_setenv ("XAUTHORITY", d->xnest_auth_file, TRUE);
+	if (SERVER_IS_PROXY (d)) {
+		int argc = ve_vector_len (argv);
+
+		ve_unsetenv ("DISPLAY");
+		if (d->parent_auth_file != NULL)
+			ve_setenv ("XAUTHORITY", d->parent_auth_file, TRUE);
 		else
 			ve_unsetenv ("XAUTHORITY");
 
-		/* Add -fp with the current font path, but only if not
-		 * already among the arguments */
-		if (strstr (command, "-fp") == NULL)
-			font_path = get_font_path (d->xnest_disp);
-		if (font_path != NULL) {
-			int argc = ve_vector_len (argv);
-			argv = g_renew (char *, argv, argc + 3);
-			argv[argc++] = "-fp";
-			argv[argc++] = font_path;
-			argv[argc++] = NULL;
-			command = g_strconcat (command, " -fp ",
-					       font_path, NULL);
+		if (d->type == TYPE_FLEXI_XNEST) {
+			char *font_path = NULL;
+			/* Add -fp with the current font path, but only if not
+			 * already among the arguments */
+			if (strstr (command, "-fp") == NULL)
+				font_path = get_font_path (d->parent_disp);
+			if (font_path != NULL) {
+				argv = g_renew (char *, argv, argc + 2);
+				argv[argc++] = "-fp";
+				argv[argc++] = font_path;
+				command = g_strconcat (command, " -fp ",
+						       font_path, NULL);
+			}
 		}
+
+		argv = g_renew (char *, argv, argc + 3);
+		argv[argc++] = "-display";
+		argv[argc++] = d->parent_disp;
+		argv[argc++] = NULL;
+		command = g_strconcat (command, " -display ",
+				       d->parent_disp, NULL);
 	}
 
 	if (argv[0] == NULL) {
@@ -1230,9 +1292,9 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 
 	gdm_fdprintf (2, "GDM: Xserver not found: %s\n"
 		      "Error: Command could not be executed!\n"
-		      "Please install the X server or edit %s to point "
-		      "to the right place.",
-		      command, GDM_CONFIG_FILE);
+		      "Please install the X server or correct "
+		      "GDM configuration and restart GDM.",
+		      command);
 
 	gdm_error (_("%s: Xserver not found: %s"), 
 		   "gdm_server_spawn", command);
@@ -1273,8 +1335,6 @@ gdm_server_usr1_handler (gint sig)
     d->servstat = SERVER_RUNNING; /* Server ready to accept connections */
     d->starttime = time (NULL);
 
-    gdm_debug ("gdm_server_usr1_handler: Got SIGUSR1, server running");
-
     server_signal_notified = TRUE;
     /* this will quit the select */
     VE_IGNORE_EINTR (write (server_signal_pipe[1], "Yay!", 4));
@@ -1295,8 +1355,6 @@ gdm_server_child_handler (int signal)
 {
 	gdm_in_signal++;
 
-	gdm_debug ("gdm_server_child_handler: Got SIGCHLD");
-
 	/* go to the main child handler */
 	gdm_slave_child_handler (signal);
 
@@ -1346,8 +1404,8 @@ gdm_server_alloc (gint id, const gchar *command)
     d->servstat = SERVER_DEAD;
     d->sesspid = 0;
     d->slavepid = 0;
-    d->type = TYPE_LOCAL;
-    d->console = TRUE;
+    d->type = TYPE_STATIC;
+    d->attached = TRUE;
     d->sessionid = 0;
     d->acctime = 0;
     d->dsp = NULL;
@@ -1385,43 +1443,42 @@ gdm_server_alloc (gint id, const gchar *command)
 }
 
 void
-gdm_server_whack_clients (GdmDisplay *disp)
+gdm_server_whack_clients (Display *dsp)
 {
 	int i, screen_count;
 	int (* old_xerror_handler) (Display *, XErrorEvent *);
 
-	if (disp == NULL ||
-	    disp->dsp == NULL)
+	if (dsp == NULL)
 		return;
 
 	old_xerror_handler = XSetErrorHandler (ignore_xerror_handler);
 
-	XGrabServer (disp->dsp);
+	XGrabServer (dsp);
 
-	screen_count = ScreenCount (disp->dsp);
+	screen_count = ScreenCount (dsp);
 
 	for (i = 0; i < screen_count; i++) {
 		Window root_ret, parent_ret;
 		Window *childs = NULL;
 		unsigned int childs_count = 0;
-		Window root = RootWindow (disp->dsp, i);
+		Window root = RootWindow (dsp, i);
 
-		while (XQueryTree (disp->dsp, root, &root_ret, &parent_ret,
+		while (XQueryTree (dsp, root, &root_ret, &parent_ret,
 				   &childs, &childs_count) &&
 		       childs_count > 0) {
 			int ii;
 
 			for (ii = 0; ii < childs_count; ii++) {
-				XKillClient (disp->dsp, childs[ii]);
+				XKillClient (dsp, childs[ii]);
 			}
 
 			XFree (childs);
 		}
 	}
 
-	XUngrabServer (disp->dsp);
+	XUngrabServer (dsp);
 
-	XSync (disp->dsp, False);
+	XSync (dsp, False);
 	XSetErrorHandler (old_xerror_handler);
 }
 
