@@ -35,6 +35,9 @@
 #include <syslog.h>
 #include "gdmwm.h"
 #include "gdm.h"
+#include "gdmcommon.h"
+#include "vicious.h"
+#include "ve-miscui.h"
 
 typedef struct _GdmWindow GdmWindow;
 struct _GdmWindow {
@@ -1242,34 +1245,6 @@ static GSourceFuncs event_funcs = {
 	event_dispatch
 };
 
-#if 0
-/* ALEX: WTF is this? It's not used by anything, and I don't want
- * the greeter to depend on libgnomeui and libbonoboui.
- */
-
-/* HAAAAAAAAAAAAAACK */
-void
-bonobo_dock_item_grab_pointer (BonoboDockItem *item)
-{
-  GdkCursor *fleur;
-
-  fleur = gdk_cursor_new (GDK_FLEUR);
-
-  /* Hm, not sure this is the right thing to do, but it seems to work.  */
-  while (gdk_pointer_grab (item->bin_window,
-                           FALSE,
-                           (GDK_BUTTON1_MOTION_MASK |
-                            GDK_POINTER_MOTION_HINT_MASK |
-                            GDK_BUTTON_RELEASE_MASK),
-                           NULL,
-                           fleur,
-                           GDK_CURRENT_TIME) != 0)
-	  gtk_main_iteration ();
-
-  gdk_cursor_destroy (fleur);
-}
-#endif
-
 void
 gdm_wm_init (Window login_window)
 {
@@ -1550,4 +1525,136 @@ gdm_wm_restore_wm_order (void)
 	}
 }
 
+void
+gdm_common_show_info_msg (const gchar *msg_file,
+			  const gchar *msg_font)
+{
+	GtkWidget *dialog, *label;
+	gchar *InfoMsg;
+	gsize InfoMsgLength;
+
+	if (ve_string_empty (msg_file) ||
+	    ! g_file_test (msg_file, G_FILE_TEST_EXISTS) ||
+	    ! g_file_get_contents (msg_file, &InfoMsg, &InfoMsgLength, NULL))
+		return;
+
+	if (InfoMsgLength <= 0) {
+		g_free (InfoMsg);
+		return;
+	}
+
+	gdm_wm_focus_new_windows (TRUE);
+	dialog = gtk_dialog_new_with_buttons (NULL /* Message */,
+					      NULL /* parent */, GTK_DIALOG_MODAL |
+					      GTK_DIALOG_DESTROY_WITH_PARENT,
+					      GTK_STOCK_OK, GTK_RESPONSE_OK,
+					      NULL);
+	label = gtk_label_new (InfoMsg);
+
+	if (msg_font && strlen (msg_font) > 0) {
+		PangoFontDescription *GdmInfoMsgFontDesc = pango_font_description_from_string (msg_font);
+		if (GdmInfoMsgFontDesc) {
+			gtk_widget_modify_font (label, GdmInfoMsgFontDesc);
+			pango_font_description_free (GdmInfoMsgFontDesc);
+		}
+	}
+
+	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), label);
+	gtk_widget_show_all (dialog);
+	gdm_wm_center_window (GTK_WINDOW (dialog));
+
+	gdm_common_setup_cursor (GDK_LEFT_PTR);
+
+	gdm_wm_no_login_focus_push ();
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	gdm_wm_no_login_focus_pop ();
+
+	g_free (InfoMsg);
+}
+
+void
+gdm_common_message (const gchar *msg)
+{
+	GtkWidget *req = NULL;
+
+	/* we should be now fine for focusing new windows */
+	gdm_wm_focus_new_windows (TRUE);
+
+	req = ve_hig_dialog_new (NULL /* parent */,
+				 GTK_DIALOG_MODAL /* flags */,
+				 GTK_MESSAGE_INFO,
+				 GTK_BUTTONS_OK,
+				 FALSE /* markup */,
+				 msg,
+				 /* avoid warning */ "%s", "");
+
+	gdm_wm_center_window (GTK_WINDOW (req));
+
+	gdm_wm_no_login_focus_push ();
+	gtk_dialog_run (GTK_DIALOG (req));
+	gtk_widget_destroy (req);
+	gdm_wm_no_login_focus_pop ();
+}
+
+gint
+gdm_common_query (const gchar *msg,
+		  gboolean markup,
+		  const char *posbutton,
+		  const char *negbutton,
+		  gboolean has_cancel)
+{
+	int ret;
+	GtkWidget *req;
+	GtkWidget *button;
+
+	/* we should be now fine for focusing new windows */
+	gdm_wm_focus_new_windows (TRUE);
+
+	req = ve_hig_dialog_new (NULL /* parent */,
+				 GTK_DIALOG_MODAL /* flags */,
+				 GTK_MESSAGE_QUESTION,
+				 GTK_BUTTONS_NONE,
+				 markup,
+				 msg,
+				 /* avoid warning */ "%s", "");
+
+	if (negbutton != NULL) {
+		button = gtk_button_new_from_stock (negbutton);
+		gtk_dialog_add_action_widget (GTK_DIALOG (req), button, GTK_RESPONSE_NO);
+		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+		gtk_widget_show (button);
+	}
+
+	if (posbutton != NULL) {
+		button = gtk_button_new_from_stock (posbutton);
+		gtk_dialog_add_action_widget (GTK_DIALOG (req), button, GTK_RESPONSE_YES);
+		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+		gtk_widget_show (button);
+	}
+
+	if (has_cancel == TRUE) {
+		button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+		gtk_dialog_add_action_widget (GTK_DIALOG (req), button, GTK_RESPONSE_CANCEL);
+		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+		gtk_widget_show (button);
+	}
+
+	if (posbutton != NULL)
+		gtk_dialog_set_default_response (GTK_DIALOG (req), GTK_RESPONSE_YES);
+	else if (negbutton != NULL)
+		gtk_dialog_set_default_response (GTK_DIALOG (req), GTK_RESPONSE_NO);
+	else if (has_cancel)
+		gtk_dialog_set_default_response (GTK_DIALOG (req), GTK_RESPONSE_CANCEL);
+
+	gdm_wm_center_window (GTK_WINDOW (req));
+
+	gdm_wm_no_login_focus_push ();
+	ret = gtk_dialog_run (GTK_DIALOG (req));
+	gdm_wm_no_login_focus_pop ();
+	gtk_widget_destroy (req);
+
+	return ret;
+}
+
 /* EOF */
