@@ -1,3 +1,21 @@
+/* GDM - The GNOME Display Manager
+ * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 #include "config.h"
 
 #include <gtk/gtk.h>
@@ -33,10 +51,7 @@ greeter_item_timed_update (void)
 static gboolean
 gdm_timer (gpointer data)
 {
-  if ((greeter_current_delay % 5) == 0)
-    {
-      greeter_item_timed_update ();
-    }
+  greeter_item_timed_update ();
   greeter_current_delay --;
   if ( greeter_current_delay <= 0 )
     {
@@ -112,8 +127,8 @@ greeter_item_timed_start (void)
       GdmTimedLoginDelay > 0)
     {
       greeter_current_delay = GdmTimedLoginDelay;
-      timed_handler_id = gtk_timeout_add (1000,
-					  gdm_timer, NULL);
+      timed_handler_id = g_timeout_add (1000,
+					gdm_timer, NULL);
     }
 }
 
@@ -122,7 +137,7 @@ greeter_item_timed_stop (void)
 {
   if (timed_handler_id != 0)
     {
-      gtk_timeout_remove (timed_handler_id);
+      g_source_remove (timed_handler_id);
       timed_handler_id = 0;
     }
 }
